"""
Compilers are modules that provide a higher order functions 
typically named "compile". The function takes a query and 
returns a function/callable  when invoked with a list of params
will apply the operations described by the query (or their 
equivalent) and return the resulting relation.

def complie(query):
  return execute(*params):
  # code ...
  return execute
"""
